#!/usr/bin/python3

import argparse, importlib, os, pkg_resources, sys
from random import randrange, seed


def main_with_defaults():
    '''To be called from zipapp'''
    args = parse_args()
    if not os.path.exists(args.answercode_filepath):
        print('No {} file found, now generating an initial one.'.format(args.answercode_filepath))
        print('...')
        with open(args.answercode_filepath, 'wb+') as new_answercode_file:
            with pkg_resources.resource_stream(__name__, 'answer_empty.py') as empty_answer_file:
                new_answercode_file.write(empty_answer_file.read())
        print('Done. You can edit the newly created file and then re-execute this.')
        return
    try:
        main(Runner(), answercode_filepath=args.answercode_filepath)
    except BaseException as exception:
        error_msg = exception.__class__.__name__
        if args.explicit:
            error_msg += ': {}'.format(exception)
        print(error_msg, file=sys.stderr)
        sys.exit(1)

def parse_args():
    parser = argparse.ArgumentParser(prog='BlackBoxChallenge01')
    parser.add_argument('--explicit', action='store_true', help='Makes it easier by displaying exceptions detailed messages')
    parser.add_argument('--answercode-filepath', default='answer.py', help='Absolute or relative path to the Python source code of the answer')
    return parser.parse_args()


def main(runner, answercode_filepath='answer.py'):
    if 'SEED' in os.environ:
        seed(int(os.environ['SEED']))
    dirname, basename = os.path.split(answercode_filepath)
    sys.path.insert(0, dirname)
    modulename, _ = os.path.splitext(basename)
    answer_module = importlib.import_module(modulename)
    amount = None
    while not runner.answer_won:
        picked = answer_module.answer(amount or runner.amount)
        amount = runner.iteration(picked)
    print('Congratulations: YOU WIN !')


class Runner:
    def __init__(self):
        self.answer_won = False
        self.amount = 0
        # We ensure the player has an initial chance to win:
        while self.amount % 4 == 0:
            self.amount = randrange(15, 32)
    def iteration(self, picked):
        if not 1 <= picked <= 3:
            raise GameOver('Invalid amount')
        self.amount -= picked
        if self.amount == 0:
            self.answer_won = True
            return 0
        if self.amount <= 3:
            raise GameOver('Your opponent won')
        if self.amount % 4 != 0:
            # Optimal strategy:
            runner_pick = self.amount - 4 * (self.amount // 4)
        else:
            runner_pick = randrange(1, 4)
        print('You picked {}, I am picking {}'.format(picked, runner_pick))
        self.amount -= runner_pick
        return self.amount

class GameOver(Exception):
    pass


if __name__ == '__main__':
    main(Runner())
